@@ -8,6 +8,7 @@
 #include "profile.h"
 #include "usb.h"
 #include "keymap_patch.h"
+#include <ckbnextconfig.h>
 
 // Values taken from the official website
 // Mice not in the list default to 12000 in the GUI
@@ -22,15 +23,17 @@ dpi_list mouse_dpi_list[] = {
     { P_SABRE_N, 10000 },
     { P_SCIMITAR, 12000 },
     { P_SCIMITAR_PRO, 16000 },
+    { P_SCIMITAR_ELITE, 18000 },
     { P_SABRE_O2, 6400 },
     { P_HARPOON, 6000 },
     { P_HARPOON_PRO, 12000 },
     { P_KATAR, 8000 },
     { P_IRONCLAW, 18000 },
+    { P_NIGHTSWORD, 18000},
     { 0, 0 }, // Keep last and do not remove
 };
 
-// WARNING: macOS doesn't support iVendor != 0x1b1c at the moment
+// WARNING: macOS doesn't support idVendor != 0x1b1c at the moment
 device_desc models[] = {
     // Keyboards
     { V_CORSAIR, P_K55, },
@@ -70,11 +73,13 @@ device_desc models[] = {
     { V_CORSAIR, P_SABRE_N, },
     { V_CORSAIR, P_SCIMITAR, },
     { V_CORSAIR, P_SCIMITAR_PRO, },
+    { V_CORSAIR, P_SCIMITAR_ELITE, },
     { V_CORSAIR, P_SABRE_O2, },
     { V_CORSAIR, P_HARPOON, },
     { V_CORSAIR, P_HARPOON_PRO, },
     { V_CORSAIR, P_KATAR, },
     { V_CORSAIR, P_IRONCLAW, },
+    { V_CORSAIR, P_NIGHTSWORD, },
     // Mousepads
     { V_CORSAIR, P_POLARIS, },
     // Headset stands
@@ -83,6 +88,7 @@ device_desc models[] = {
 
 size_t N_MODELS = sizeof(models) / sizeof(device_desc);
 
+device_desc ignored_devices[DEV_MAX] = {0};
 
 /// brief .
 ///
@@ -166,7 +172,7 @@ const char* product_str(ushort product){
         return "m65e";
     if(product == P_SABRE_O || product == P_SABRE_L || product == P_SABRE_N || product == P_SABRE_O2)
         return "sabre";
-    if(product == P_SCIMITAR || product == P_SCIMITAR_PRO)
+    if(product == P_SCIMITAR || product == P_SCIMITAR_PRO || product == P_SCIMITAR_ELITE)
         return "scimitar";
     if(product == P_HARPOON || product == P_HARPOON_PRO)
         return "harpoon";
@@ -176,6 +182,8 @@ const char* product_str(ushort product){
         return "katar";
     if(product == P_IRONCLAW)
         return "ironclaw";
+    if(product == P_NIGHTSWORD)
+        return "nightsword";
     if(product == P_POLARIS)
         return "polaris";
     if(product == P_ST100)
@@ -261,11 +269,11 @@ static void* devmain(usbdevice* kb){
         /// (checked via IS_CONNECTED(kb)).
         /// This is true if the kb-structure has a handle and an event pointer both != Null).
         /// If not, the loop is left (the first exit point).
-        pthread_mutex_unlock(dmutex(kb));
+        queued_mutex_unlock(dmutex(kb));
         // Read from FIFO
         const char* line;
         int lines = readlines(kbfifo, linectx, &line);
-        pthread_mutex_lock(dmutex(kb));
+        queued_mutex_lock(dmutex(kb));
         // End thread when the handle is removed
         if(!IS_CONNECTED(kb))
             break;
@@ -287,7 +295,7 @@ static void* devmain(usbdevice* kb){
             }
         }
     }
-    pthread_mutex_unlock(dmutex(kb));
+    queued_mutex_unlock(dmutex(kb));
     ///
     /// After leaving the endless loop the readlines-ctx structure and its buffers are freed by readlines_ctx_free().
     readlines_ctx_free(linectx);
@@ -336,7 +344,8 @@ static void* _setupusb(void* context){
     /// - the standard delay time is initialized in kb->usbdelay
     ///
     usbdevice* kb = context;
-    pthread_mutex_lock(imutex(kb));
+    queued_mutex_lock(dmutex(kb));
+    queued_mutex_lock(imutex(kb));
     // Set standard fields
     ushort vendor = kb->vendor, product = kb->product;
     const devcmd* vt = kb->vtable = get_vtable(vendor, product);
@@ -350,7 +359,7 @@ static void* _setupusb(void* context){
     /// Allocate memory for the os_usbrecv() buffer
     kb->interruptbuf = malloc(MSG_SIZE * sizeof(uchar));
     if(!kb->interruptbuf)
-        ckb_fatal("Error allocating memory for usb_recv() %s\n", strerror(errno));
+        ckb_fatal("Error allocating memory for usb_recv() %s", strerror(errno));
 
     // Check if the device needs a patched keymap, and if so patch it.
     patchkeys(kb);
@@ -430,21 +439,10 @@ static void* _setupusb(void* context){
     /// However, since they allocate storage areas, the subsequent assignments and initializations can run in a SEGV.
     vt->allocprofile(kb);
     ///
-    /// - Not completely understandable is why now via the vtable the function updateindicators() is called.
-    /// But this actually happens in the just started thread _ledthread().
-    /// Either the initialization is wrong und must done here with force or the overview is lost, what happens when...\n
-    /// Regardless: For a mouse nothing happens here, for a keyboard updateindicators_kb() is called via the entry in kb->vtable.
-    /// The first parameter is kb again, the second is constant 1 (means force = true).
-    /// This causes the LED status to be sent after a 5ms delay via os_sendindicators()
-    /// (ioctl with a \c usbdevfs_ctrltransfer).
-    /// \n The notification is sent to all currently open notification channels then.
-    /// \n Setupindicators() and with it updateindicators_kb() can fail.
-    vt->updateindicators(kb, 1);
-    ///
     /// - From this point - if an error is detected - the error label is addressed by goto statement,
     /// which first performs an unlock on the imutex.
     /// This is interesting because the next statement is exactly this: An unlock on the imutex.
-    pthread_mutex_unlock(imutex(kb));
+    queued_mutex_unlock(imutex(kb));
     ///
     /// - Via vtable the \a kb->start() function is called next.
     /// This is the same for a mouse and an RGB keyboard: start_dev(),
@@ -495,18 +493,20 @@ static void* _setupusb(void* context){
     /// and devmain() is called.
     // Finished. Enter main loop
     int index = INDEX_OF(kb, keyboard);
-    ckb_info("Setup finished for %s%d\n", devpath, index);
-    updateconnected();
+    ckb_info("Setup finished for %s%d", devpath, index);
+    queued_mutex_unlock(dmutex(kb));
+    updateconnected(kb);
+    queued_mutex_lock(dmutex(kb));
     ///
     /// devmain()'s return value is returned by _setupusb() when we terminate.
     return devmain(kb);
     ///
     /// - The remaining code lines are the two exit labels as described above
     fail:
-    pthread_mutex_unlock(imutex(kb));
+    queued_mutex_unlock(imutex(kb));
     fail_noinput:
     closeusb(kb);
-    pthread_mutex_unlock(dmutex(kb));
+    queued_mutex_unlock(dmutex(kb));
     return 0;
 }
 
@@ -519,7 +519,7 @@ static void* _setupusb(void* context){
 void setupusb(usbdevice* kb){
 
     if(pthread_create(&kb->thread, 0, _setupusb, kb)) {
-        ckb_err("Failed to create USB thread\n");
+        ckb_err("Failed to create USB thread");
         return;
     }
 
@@ -530,6 +530,7 @@ void setupusb(usbdevice* kb){
     usbthread_name[3] = INDEX_OF(kb, keyboard) + '0';
     pthread_setname_np(kb->thread, usbthread_name);
 #endif // OS_MAC
+    queued_mutex_unlock(dmutex(kb));
 }
 
 /// \brief .
@@ -608,17 +609,17 @@ int _resetusb(usbdevice* kb, const char* file, int line){
 int usb_tryreset(usbdevice* kb){
     if(reset_stop)
         return -1;
-    ckb_info("Attempting reset...\n");
+    ckb_info("Attempting reset...");
     while(1){
         int res = resetusb(kb);
         if(!res){
-            ckb_info("Reset success\n");
+            ckb_info("Reset success");
             return 0;
         }
         if(res == -2 || reset_stop)
             break;
     }
-    ckb_err("Reset failed. Disconnecting.\n");
+    ckb_err("Reset failed. Disconnecting.");
     return -1;
 }
 
@@ -677,10 +678,10 @@ int _usbsend(usbdevice* kb, const uchar* messages, int count, const char* file,
     for(int i = 0; i < count; i++){
         // Send each message via the OS function
         while(1){
-            pthread_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
             DELAY_SHORT(kb);
+            queued_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
             int res = os_usbsend(kb, messages + i * MSG_SIZE, 0, file, line);
-            pthread_mutex_unlock(mmutex(kb));
+            queued_mutex_unlock(mmutex(kb));
             if(res == 0)
                 return 0;
             else if(res != -1){
@@ -699,9 +700,10 @@ int _usbsend(usbdevice* kb, const uchar* messages, int count, const char* file,
 
 int _usbsend_control(usbdevice* kb, uchar* data, ushort len, uchar bRequest, ushort wValue, ushort wIndex, const char* file, int line){
     while(1){
-        pthread_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
+        DELAY_SHORT(kb);
+        queued_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
         int res = os_usbsend_control(kb, data, len, bRequest, wValue, wIndex, file, line);
-        pthread_mutex_unlock(mmutex(kb));
+        queued_mutex_unlock(mmutex(kb));
 
         if(res != -1)
             return res;
@@ -763,10 +765,10 @@ int _usbrecv(usbdevice* kb, const uchar* out_msg, uchar* in_msg, const char* fil
     // Try a maximum of 5 times
     for (int try = 0; try < 5; try++) {
         // Send the output message
-        pthread_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
+        queued_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
         DELAY_SHORT(kb);
         int res = os_usbsend(kb, out_msg, 1, file, line);
-        pthread_mutex_unlock(mmutex(kb));
+        queued_mutex_unlock(mmutex(kb));
         if (res == 0)
             return 0;
         else if (res == -1) {
@@ -777,7 +779,8 @@ int _usbrecv(usbdevice* kb, const uchar* out_msg, uchar* in_msg, const char* fil
             continue;
         }
         // Wait for the response
-        DELAY_MEDIUM(kb);
+        if(!(kb->fwversion >= 0x120 || IS_V2_OVERRIDE(kb)))
+            DELAY_MEDIUM(kb);
         res = os_usbrecv(kb, in_msg, file, line);
         if(res == 0)
             return 0;
@@ -785,10 +788,11 @@ int _usbrecv(usbdevice* kb, const uchar* out_msg, uchar* in_msg, const char* fil
             return res;
         if(reset_stop || hwload_mode != 2)
             return 0;
-        DELAY_LONG(kb);
+        if(!(kb->fwversion >= 0x120 || IS_V2_OVERRIDE(kb)))
+            DELAY_LONG(kb);
     }
     // Give up
-    ckb_err_fn("Too many send/recv failures. Dropping.\n", file, line);
+    ckb_err_fn("Too many send/recv failures. Dropping.", file, line);
     return 0;
 }
 
@@ -836,23 +840,47 @@ int _usbrecv(usbdevice* kb, const uchar* out_msg, uchar* in_msg, const char* fil
 /// and closeusb() always returns zero (success).
 ///
 int closeusb(usbdevice* kb){
-    pthread_mutex_lock(imutex(kb));
+    queued_mutex_lock(imutex(kb));
     if(kb->handle){
         int index = INDEX_OF(kb, keyboard);
-        ckb_info("Disconnecting %s%d\n", devpath, index);
+        ckb_info("Disconnecting %s%d", devpath, index);
         os_inputclose(kb);
-        updateconnected();
+        queued_mutex_unlock(imutex(kb));
+        queued_mutex_unlock(dmutex(kb));
+        updateconnected(kb);
+        queued_mutex_lock(dmutex(kb));
+        queued_mutex_lock(imutex(kb));
         // Close USB device
         os_closeusb(kb);
-    } else
-        updateconnected();
+    } else {
+        queued_mutex_unlock(imutex(kb));
+        queued_mutex_unlock(dmutex(kb));
+        updateconnected(kb);
+        queued_mutex_lock(dmutex(kb));
+        queued_mutex_lock(imutex(kb));
+    }
     rmdevpath(kb);
 
     // Wait for thread to close
-    pthread_mutex_unlock(imutex(kb));
-    pthread_mutex_unlock(dmutex(kb));
-    pthread_join(kb->thread, 0);
-    pthread_mutex_lock(dmutex(kb));
+    queued_mutex_unlock(imutex(kb));
+    queued_mutex_unlock(dmutex(kb));
+
+    if(pthread_equal(pthread_self(), kb->thread)){
+#ifdef DEBUG_MUTEX
+        ckb_info("Attempted to pthread_join() self. Detaching instead.");
+#endif
+        int detachres = pthread_detach(kb->thread);
+        if(detachres)
+            ckb_err("pthread_detach() returned %s (%d)", strerror(detachres), detachres);
+    } else {
+#ifdef DEBUG_MUTEX
+        ckb_info("Joining thread 0x%lx for ckb%d by thread 0x%lx", kb->thread, INDEX_OF(kb, keyboard), pthread_self());
+#endif
+        int joinres = pthread_join(kb->thread, NULL);
+        if(joinres)
+            ckb_err("pthread_join() returned %s (%d)", strerror(joinres), joinres);
+    }
+    queued_mutex_lock(dmutex(kb));
 
     // Free the device-specific keymap
     free(kb->keymap);
@@ -864,7 +892,9 @@ int closeusb(usbdevice* kb){
     if(!kb->vtable)
         return 0;
     kb->vtable->freeprofile(kb);
+    queued_mutex_lock(imutex(kb));
     memset(kb, 0, sizeof(usbdevice));
+    queued_mutex_unlock(imutex(kb));
     return 0;
 }
 
@@ -874,26 +904,32 @@ void print_urb_buffer(const char* prefix, const unsigned char* buffer, int actua
     for(int i = 0; i < actual_length; i++)
         sprintf(&converted[i * 3], "%02x ", buffer[i]);
     if(line == 0)
-        ckb_info("ckb%i %s %s\n", devnum, prefix, converted);
+        ckb_info("ckb%i %s %s", devnum, prefix, converted);
     else
-        ckb_info("ckb%i %s (via %s:%d) %s %s\n", devnum, function, file, line, prefix, converted);
+        ckb_info("ckb%i %s (via %s:%d) %s %s", devnum, function, file, line, prefix, converted);
 }
 
-void reactivate_devices()
-{
-    ckb_info("System has woken from sleep\n");
-    usbdevice *kb = NULL;
-    for(int i = 0; i < DEV_MAX; i++){
-        if(IS_CONNECTED(keyboard + i)){
-            kb = keyboard + i;
+#define TRY_WITH_RESET_NOFREE(action)                \
+    while(action){                                   \
+        if(usb_tryreset(kb)){                        \
+            ckb_err(#action " failed after resume"); \
+            return;                                  \
+        }                                            \
+    }
+
+void reactivate_devices(){
+    ckb_info("System has woken from sleep");
+    for(int i = 1; i < DEV_MAX; i++){
+        usbdevice* kb = keyboard + i;
+        queued_mutex_lock(dmutex(kb));
+        if(IS_CONNECTED(kb) && !NEEDS_FW_UPDATE(kb) && kb->active){
             // If the device was active, mark it as disabled and re-enable it
-            pthread_mutex_lock(dmutex(kb));
-            if(kb->active){
-                kb->active = 0;
-                const devcmd* vt = kb->vtable;
-                vt->active(kb, 0, 0, 0, 0);
-            }
-            pthread_mutex_unlock(dmutex(kb));
+            kb->active = 0;
+            const devcmd* vt = kb->vtable;
+            TRY_WITH_RESET_NOFREE(vt->active(kb, 0, 0, 0, 0));
+            TRY_WITH_RESET_NOFREE(vt->updatergb(kb, 1));
+            TRY_WITH_RESET_NOFREE(vt->updatedpi(kb, 1));
         }
+        queued_mutex_unlock(dmutex(kb));
     }
 }
