@@ -14,7 +14,7 @@ static QSet<QString> notifyPaths;
 static QMutex notifyPathMutex;
 
 int Kb::_frameRate = 30, Kb::_scrollSpeed = 0;
-bool Kb::_dither = false, Kb::_mouseAccel = true, Kb::_delay = false;
+bool Kb::_dither = false, Kb::_mouseAccel = true;
 
 Kb::Kb(QObject *parent, const QString& path) :
     QThread(parent), features("N/A"), firmware("N/A"), pollrate("N/A"), monochrome(false), hwload(false), adjrate(false),
@@ -139,7 +139,6 @@ Kb::Kb(QObject *parent, const QString& path) :
     // Activate device, apply settings, and ask for hardware profile
     cmd.write(QString("fps %1\n").arg(_frameRate).toLatin1());
     cmd.write(QString("dither %1\n").arg(static_cast<int>(_dither)).toLatin1());
-    cmd.write(QString("\ndelay %1\n").arg(_delay? "on" : "off").toLatin1());
 #ifdef Q_OS_MACOS
     // Write ANSI/ISO flag to daemon (OSX only)
     cmd.write("layout ");
@@ -297,6 +296,8 @@ void Kb::load(){
             demoProfile = ":/txt/demoprofile_polaris.ini";
         else if(map.model() == KeyMap::ST100)
             demoProfile = ":/txt/demoprofile_st100.ini";
+        else if(map.model() == KeyMap::NIGHTSWORD)
+            demoProfile = ":/txt/demoprofile_nightsword.ini";
         QSettings demoSettings(demoProfile, QSettings::IniFormat, this);
         CkbSettings cSettings(demoSettings);
         KbProfile* demo = new KbProfile(this, map, cSettings, "{BA7FC152-2D51-4C26-A7A6-A036CC93D924}");
@@ -451,14 +452,12 @@ void Kb::frameUpdate(){
         index = 3 + index % 3;
 
     // Send lighting/binding to driver
-    cmd.write(QString("mode %1 switch ").arg(index + 1).toLatin1());
+    if(prevMode != _currentMode || changed)
+        cmd.write(QString("mode %1 switch ").arg(index + 1).toLatin1());
     perf->applyIndicators(index, iState);
     light->frameUpdate(cmd, monochrome);
-    cmd.write(QString("\n@%1 ").arg(notifyNumber).toLatin1());
-    bind->update(cmd, changed);
-    cmd.write(" ");
+    bind->update(cmd, notifyNumber, changed);
     perf->update(cmd, notifyNumber, changed, true);
-    cmd.write("\n");
     cmd.flush();
 }
 
@@ -572,8 +571,6 @@ void Kb::readNotify(QString line){
         }
         hwProfile(newProfile);
         emit profileAdded();
-        if(_hwProfile == _currentProfile)
-            emit profileChanged();
     } else if(components[0] == "hwprofilename"){
         // Hardware profile name
         QString name = QUrl::fromPercentEncoding(components[1].toUtf8());
@@ -650,8 +647,6 @@ void Kb::readNotify(QString line){
             if(!(oldName.length() >= name.length() && oldName.left(name.length()) == name)){
                 // Don't change the name if it's a truncated version of what we already have
                 hwMode->name(name);
-                if(_hwProfile == _currentProfile)
-                    emit modeRenamed();
             }
         } else if(components[2] == "hwrgb"){
             // RGB - set mode lighting
@@ -684,7 +679,6 @@ void Kb::readNotify(QString line){
                                 lightColor = color;
                             if(key.startsWith("dpi") && key.length() > 3){
                                 // DPI levels go to the KbPerf object instead of KbLight
-                                bool ok = false;
                                 int index = key.mid(3).toInt(&ok);
                                 if(ok)
                                     kbmode->perf()->dpiColor(index, color);
@@ -797,42 +791,23 @@ KeyMap Kb::getKeyMap(){
     return KeyMap(_model, _layout);
 }
 
-void Kb::setCurrentProfile(KbProfile *profile, bool spontaneous){
+void Kb::setCurrentProfile(KbProfile* profile){
     while(profile->modeCount() < hwModeCount)
         profile->append(new KbMode(this, getKeyMap()));
-    KbMode* mode = profile->currentMode();
-    if(!mode)
-        profile->currentMode(mode = profile->modes().first());
-    setCurrentMode(profile, mode, spontaneous);
+
+    emit profileAboutToChange();
+    _currentProfile = profile;
+    emit profileChanged();
+    setCurrentMode(profile->currentMode());
 }
 
-void Kb::setCurrentMode(KbProfile* profile, KbMode* mode, bool spontaneous){
-    if(_currentProfile != profile){
-        _currentProfile = profile;
-        _needsSave = true;
-        emit profileChanged();
-    }
-    if(_currentMode != mode || _currentProfile->currentMode() != mode){
-        _currentProfile->currentMode(_currentMode = mode);
-        _needsSave = true;
-        emit modeChanged(spontaneous);
-    }
+void Kb::setCurrentMode(KbMode* mode){
+    _currentProfile->currentMode(_currentMode = mode);
+    _needsSave = true;
+    emit modeChanged();
     mode->light()->forceFrameUpdate();
 }
 
-////
-/// \brief Kb::macroDelay handles the UI-Element macroBox.
-/// Sends a command to the keyboard to switch on or off the delay function on very large macros
-/// \param flag true: Switch on delay function, else switch off
-///
-void Kb::macroDelay(bool flag) {
-   _delay = flag;
-
-   foreach(Kb* kb, activeDevices){
-       kb->cmd.write(QString("\ndelay %1\n").arg(flag? "on" : "off").toLatin1());
-   }
-}
-
 KeyMap::Layout Kb::getCurrentLayout(){
     return _layout;
 }
