@@ -63,7 +63,7 @@ void KbProfileDialog::repopulate(){
 
 void KbProfileDialog::addNewProfileItem(){
     // Add an item for creating a new profile. Make it editable but not dragable.
-    QListWidgetItem* item = new QListWidgetItem("New profile...", ui->profileList);
+    QListWidgetItem* item = new QListWidgetItem(tr("New profile..."), ui->profileList);
     item->setFlags((item->flags() | Qt::ItemIsEditable) & ~Qt::ItemIsDragEnabled & ~Qt::ItemIsDropEnabled);
     item->setData(NEW_FLAG, 1);
     QFont font = item->font();
@@ -113,23 +113,23 @@ void KbProfileDialog::on_profileList_customContextMenuRequested(const QPoint &po
     QList<KbProfile*> profiles = device->profiles();
 
     QMenu menu(this);
-    QAction* rename = new QAction("Rename", this);
-    QAction* duplicate = new QAction("Duplicate", this);
-    QAction* del = new QAction("Delete", this);
+    QAction* rename = new QAction(tr("Rename"), this);
+    QAction* duplicate = new QAction(tr("Duplicate"), this);
+    QAction* del = new QAction(tr("Delete"), this);
     bool canDelete = (profiles.count() > 1);
     if(!canDelete)
         // Can't delete the last profile on the device
         del->setEnabled(false);
-    QAction* hwsave = new QAction("Save to Hardware", this);
+    QAction* hwsave = new QAction(tr("Save to Hardware"), this);
     // Disable Save to hardware button for unsupported devices
     if(!device->hwload){
         hwsave->setDisabled(true);
-        hwsave->setToolTip(QString(tr("Saving to hardware is not supported on this device.")));
+        hwsave->setToolTip(tr("Saving to hardware is not supported on this device."));
     }
-    QAction* moveup = new QAction("Move Up", this);
+    QAction* moveup = new QAction(tr("Move Up"), this);
     if(index == 0)
         moveup->setEnabled(false);
-    QAction* movedown = new QAction("Move Down", this);
+    QAction* movedown = new QAction(tr("Move Down"), this);
     if(index >= profiles.count() - 1)
         movedown->setEnabled(false);
     menu.addAction(rename);
@@ -169,11 +169,11 @@ void KbProfileDialog::on_profileList_customContextMenuRequested(const QPoint &po
         // Refresh item icons
         int count = ui->profileList->count();
         for(int i = 0; i < count; i++){
-            QListWidgetItem* item = ui->profileList->item(i);
-            if(item->data(NEW_FLAG).toUInt() == 1)
+            QListWidgetItem* hwitem = ui->profileList->item(i);
+            if(hwitem->data(NEW_FLAG).toUInt() == 1)
                 continue;
-            KbProfile* profile = device->find(item->data(GUID).toUuid());
-            item->setIcon(QIcon((profile == device->hwProfile()) ? ":/img/icon_profile_hardware.png" : ":/img/icon_profile.png"));
+            KbProfile* profile = device->find(hwitem->data(GUID).toUuid());
+            hwitem->setIcon(QIcon((profile == device->hwProfile()) ? ":/img/icon_profile_hardware.png" : ":/img/icon_profile.png"));
         }
         // No need to repopulate here either
         return;
@@ -198,7 +198,7 @@ void KbProfileDialog::extractedFileCleanup(QStringList extracted){
 }
 
 void KbProfileDialog::on_exportButton_clicked(){
-    quint16 profileVer = CKB_NEXT_PROFILE_VER;
+    const quint16 profileVer = CKB_NEXT_PROFILE_VER;
 
     // Selected items
     QList<QListWidgetItem*> selectedItems = ui->profileList->selectedItems();
@@ -210,6 +210,22 @@ void KbProfileDialog::on_exportButton_clicked(){
     dialog.setViewMode(QFileDialog::List);
     dialog.setAcceptMode(QFileDialog::AcceptSave);
 
+    // Generate a default filename
+    QString filename;
+    int itemcount = selectedItems.count();
+    for(int i = 0; i < itemcount; i++){
+        QString pname(selectedItems.at(i)->text());
+        pname.replace(QRegularExpression("[^a-zA-Z0-9_]"), QString());
+        if(pname.isEmpty())
+            continue;
+        filename.append(pname);
+        if(i < itemcount - 1)
+            filename.append("_");
+    }
+
+    if(!filename.isEmpty())
+        dialog.selectFile(filename);
+
     if(!dialog.exec())
         return;
 
@@ -219,7 +235,7 @@ void KbProfileDialog::on_exportButton_clicked(){
         return;
 
     // Pick only the first filename
-    QString filename = filenames.at(0);
+    filename = filenames.at(0);
     if(!filename.endsWith(".ckb"))
         filename.append(".ckb");
 
@@ -259,9 +275,10 @@ void KbProfileDialog::on_exportButton_clicked(){
 
         // Used to make sure QSettings is destroyed before trying to hash the ini
         {
-            QSettings exportitem(tmp, QSettings::IniFormat);
+            // The version passed here isn't used anywhere
+            CkbExternalSettings exportitem(tmp, QSettings::IniFormat, CKB_NEXT_PROFILE_VER);
             exportitem.clear();
-            prof->profileExport(&exportitem);
+            prof->save(exportitem);
             exportitem.sync();
 
             if(exportitem.status() == QSettings::NoError)
@@ -318,8 +335,8 @@ bool KbProfileDialog::verifyHash(QString file){
     return false;
 }
 
-void KbProfileDialog::importCleanup(QStringList extracted, QList<QPair<QSettings*, QString>> profileptrs){
-    // Destroy the open QSettings objects
+void KbProfileDialog::importCleanup(QStringList extracted, QList<QPair<CkbExternalSettings*, QString>> profileptrs){
+    // Destroy the open CkbExternalSettings objects
     for(int i = 0; i < profileptrs.count(); i++)
         delete profileptrs.at(i).first;
 
@@ -329,7 +346,6 @@ void KbProfileDialog::importCleanup(QStringList extracted, QList<QPair<QSettings
 }
 
 void KbProfileDialog::on_importButton_clicked(){
-    quint16 profileVer = CKB_NEXT_PROFILE_VER;
     QFileDialog dialog(this);
     dialog.setFileMode(QFileDialog::ExistingFile);
     dialog.setNameFilter(tr("ckb-next profiles (*.ckb)"));
@@ -350,7 +366,7 @@ void KbProfileDialog::on_importButton_clicked(){
     QStringList extracted = JlCompress::extractDir(filename, QString("/tmp/"));
 
     if(extracted.empty()){
-        QMessageBox::warning(this, tr("Error"), QString(tr("Could not read %1.")).arg(filename), QMessageBox::Ok);
+        QMessageBox::warning(this, tr("Error"), tr("Could not read %1.").arg(filename), QMessageBox::Ok);
         return;
     }
 
@@ -360,13 +376,14 @@ void KbProfileDialog::on_importButton_clicked(){
         extractedFileCleanup(extracted);
         return;
     }
-
+    quint16 currentProfileVer = 0;
     {
         QSettings metadata("/tmp/ckbpkg.metadata", QSettings::IniFormat);
         metadata.beginGroup("metadata");
 
         // Pkg version check
-        if(profileVer < metadata.value("version").toInt()){
+        currentProfileVer = metadata.value("version").toInt();
+        if(CKB_NEXT_PROFILE_VER < currentProfileVer){
             QMessageBox::warning(this, tr("Error"), tr("Unsupported package selected. Please update ckb-next to import it."), QMessageBox::Ok);
             extractedFileCleanup(extracted);
             return;
@@ -399,9 +416,9 @@ void KbProfileDialog::on_importButton_clicked(){
 
             // Only identical devices are supported for now, but let the user override this if they want
             int ret = QMessageBox::question(this, tr("Profile Import"),
-                                            QString(tr("This profile was created for a %1 (%2)\nbut it is going to be imported to a %3 (%4).\n\n"
+                                                  tr("This profile was created for a %1 (%2)\nbut it is going to be imported to a %3 (%4).\n\n"
                                                        "You may need to manually add some keys to the appropriate animations.\n\n"
-                                                       "Import Anyway?")).arg(metadataDevModel, metadataDevLayoutStr, currentDevModel, currentDevLayoutStr),
+                                                       "Import Anyway?").arg(metadataDevModel, metadataDevLayoutStr, currentDevModel, currentDevLayoutStr),
                                             QMessageBox::Yes,
                                             QMessageBox::No);
 
@@ -416,7 +433,7 @@ void KbProfileDialog::on_importButton_clicked(){
 
     QStringList profilestr;
     // <Pointer>, <GUID>
-    QList<QPair<QSettings*, QString>> profileptrs;
+    QList<QPair<CkbExternalSettings*, QString>> profileptrs;
     int pkgProfileCount = 0;
     int pkgVerifiedCount = 0;
 
@@ -425,10 +442,10 @@ void KbProfileDialog::on_importButton_clicked(){
         if(tmpFile.endsWith("ini")){
             pkgProfileCount++;
             if(verifyHash(tmpFile)){
-                QSettings* sptr = new QSettings(extracted.at(i), QSettings::IniFormat);
+                CkbExternalSettings* sptr = new CkbExternalSettings(extracted.at(i), QSettings::IniFormat, currentProfileVer);
                 QString guid(sptr->childGroups().first());
 
-                profileptrs.append(QPair<QSettings*, QString>(sptr, guid));
+                profileptrs.append(QPair<CkbExternalSettings*, QString>(sptr, guid));
                 profilestr.append(sptr->value(guid + "/Name").toString());
                 pkgVerifiedCount++;
             }
@@ -447,11 +464,11 @@ void KbProfileDialog::on_importButton_clicked(){
         QMessageBox::warning(this, tr("Error"), tr("Selected package contains some corrupt profiles.\n\nAn attempt will be made to import as many as possible."), QMessageBox::Ok);
 
     QString profilestrf(profilestr.join("\n• "));
-    int ret = QMessageBox::information(this, tr("Profile Import"),
-                                   QString(tr("The following profiles will be imported.\n\n• ")).append(profilestrf),
+    int msgret = QMessageBox::information(this, tr("Profile Import"),
+                                   tr("The following profiles will be imported.\n\n• ").append(profilestrf),
                                    QMessageBox::Ok,
                                    QMessageBox::Cancel);
-    if(ret == QMessageBox::Cancel){
+    if(msgret == QMessageBox::Cancel){
         importCleanup(extracted, profileptrs);
         return;
     }
@@ -460,7 +477,7 @@ void KbProfileDialog::on_importButton_clicked(){
 
     for(int i = 0; i < profileptrs.count(); i++){
         int ret = 0;
-        QSettings* sptr = profileptrs.at(i).first;
+        CkbExternalSettings* sptr = profileptrs.at(i).first;
         //QString guid = profileptrs.at(i).second;
         //QUuid current = guid.trimmed();
         QUuid guid = sptr->childGroups().first().trimmed();
@@ -491,7 +508,6 @@ void KbProfileDialog::on_importButton_clicked(){
             profileGuid = QUuid::createUuid().toString();
 
         qDebug() << "Importing" << profname;
-
         KbProfile* newProfile = device->newProfile(sptr, profileGuid);
         profiles.append(newProfile);
     }
