@@ -3,6 +3,7 @@
 
 #include <QDialog>
 #include "kbwidget.h"
+#include <QListWidgetItem>
 
 namespace Ui {
 class KbProfileDialog;
@@ -13,13 +14,13 @@ class KbProfileDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit KbProfileDialog(KbWidget *parent = 0);
+    explicit KbProfileDialog(KbWidget* parent = 0);
     ~KbProfileDialog();
 
 private slots:
     void profileList_reordered();
-    void on_profileList_itemClicked(QListWidgetItem *item);
-    void on_profileList_itemChanged(QListWidgetItem *item);
+    void on_profileList_itemClicked(QListWidgetItem* item);
+    void on_profileList_itemChanged(QListWidgetItem* item);
     void on_profileList_customContextMenuRequested(const QPoint &pos);
     void on_exportButton_clicked();
     void on_importButton_clicked();
@@ -37,7 +38,7 @@ private:
     void addNewProfileItem();
 
     bool verifyHash(QString file);
-    void importCleanup(QStringList extracted, QList<QPair<QSettings*, QString>> profileptrs);
+    void importCleanup(QStringList extracted, QList<QPair<CkbExternalSettings*, QString> > profileptrs);
     void extractedFileCleanup(QStringList extracted);
 
 };
