@@ -15,6 +15,8 @@
 #include <sys/socket.h>
 #include <signal.h>
 #include <QProcess>
+#include "xcb/xwindowdetector.h"
+XWindowDetector* windowDetector = nullptr;
 
 #ifndef DISABLE_UPDATER
 #include "ckbupdater.h"
@@ -27,52 +29,10 @@
 extern QSharedMemory appShare;
 extern QString devpath;
 
-static const QString configLabel = "Settings";
-
 int MainWindow::signalHandlerFd[2] = {0, 0};
 
-#ifndef Q_OS_MACOS
-QString daemonDialogText = QObject::tr("Start it once with:") +
-    "<blockquote><code>sudo systemctl start ckb-next-daemon</code></blockquote>" +
-    QObject::tr("Enable it for every boot:") +
-    "<blockquote><code>sudo systemctl enable ckb-next-daemon</code></blockquote>";
-#else
-QString daemonDialogText = QObject::tr("Start and enable it with:") +
-    "<blockquote><code>sudo launchctl load -w /Library/LaunchDaemons/org.ckb-next.daemon.plist</code></blockquote>";
-#endif
-
 MainWindow* MainWindow::mainWindow = 0;
 
-#ifdef USE_LIBAPPINDICATOR
-extern "C" {
-    void quitIndicator(GtkMenu* menu, gpointer data) {
-        Q_UNUSED(menu);
-        MainWindow* window = static_cast<MainWindow*>(data);
-        window->quitApp();
-    }
-
-    void restoreIndicator(GtkMenu* menu, gpointer data) {
-        Q_UNUSED(menu);
-        MainWindow* window = static_cast<MainWindow*>(data);
-        window->showWindow();
-    }
-
-    void indicatorScroll(AppIndicator* indicator, guint steps, GdkScrollDirection direction, gpointer data) {
-        MainWindow* window = static_cast<MainWindow*>(data);
-        switch(direction) {
-            case GDK_SCROLL_UP:
-                emit window->trayIconScrolled(true);
-                break;
-            case GDK_SCROLL_DOWN:
-                emit window->trayIconScrolled(false);
-                break;
-            default:
-                break;
-        }
-    }
-}
-#endif
-
 #if defined(Q_OS_MACOS) && !defined(OS_MAC_LEGACY)
 bool is_catalina_or_higher(){
     // Get macOS version. If Catalina or higher, start the daemon agent as the current user to request for HID permission.
@@ -162,6 +122,8 @@ MainWindow::MainWindow(QWidget *parent) :
     ui->setupUi(this);
     mainWindow = this;
 
+    kbfw = new KbFirmware();
+
     // Start device manager
     KbManager::init(CKB_NEXT_VERSION_STR);
     connect(KbManager::kbManager(), SIGNAL(kbConnected(Kb*)), this, SLOT(addDevice(Kb*)));
@@ -172,61 +134,14 @@ MainWindow::MainWindow(QWidget *parent) :
     // Set up tray icon
     restoreAction = new QAction(tr("Restore"), this);
     closeAction = new QAction(tr("Quit"), this);
-    changeTrayIconAction = new QAction(tr("Monochrome Tray Icon"), this);
-
-#ifdef USE_LIBAPPINDICATOR
-    QProcessEnvironment procEnv = QProcessEnvironment::systemEnvironment();
-
-    QString desktop = procEnv.value("XDG_CURRENT_DESKTOP", QString("")).toLower();
-    QString qpaTheme = procEnv.value("QT_QPA_PLATFORMTHEME", QString("")).toLower();
-    QString ckbnextAppindicator = procEnv.value("CKB_NEXT_USE_APPINDICATOR", QString("")).toLower();
-    QStringList appIndicatorOn = QStringList() << "yes" << "on" << "1";
-    QStringList appIndicatorOff = QStringList() << "no" << "off" << "0";
-
-    useAppindicator = false;
-    trayIcon = 0;
-
-    if(((desktop == "unity" || qpaTheme == "appmenu-qt5" || qpaTheme == "gtk2") && !appIndicatorOff.contains(ckbnextAppindicator)) || appIndicatorOn.contains(ckbnextAppindicator)){
-        qDebug() << "Using AppIndicator";
-        useAppindicator = true;
-
-        indicatorMenu = gtk_menu_new();
-        indicatorMenuRestoreItem = gtk_menu_item_new_with_label("Restore");
-        indicatorMenuQuitItem = gtk_menu_item_new_with_label("Quit");
-
-        gtk_menu_shell_append(GTK_MENU_SHELL(indicatorMenu), indicatorMenuRestoreItem);
-        gtk_menu_shell_append(GTK_MENU_SHELL(indicatorMenu), indicatorMenuQuitItem);
-
-        g_signal_connect(indicatorMenuQuitItem, "activate",
-            G_CALLBACK(quitIndicator), this);
-        g_signal_connect(indicatorMenuRestoreItem, "activate",
-            G_CALLBACK(restoreIndicator), this);
-
-        gtk_widget_show(indicatorMenuRestoreItem);
-        gtk_widget_show(indicatorMenuQuitItem);
-
-        indicator = app_indicator_new("ckb-next", "indicator-messages", APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
-
-        app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ACTIVE);
-        app_indicator_set_menu(indicator, GTK_MENU(indicatorMenu));
-        app_indicator_set_icon(indicator, "ckb-next");
-
-        // Catch scroll events
-        g_signal_connect(indicator, "scroll-event", G_CALLBACK(indicatorScroll), this);
-    } else
-#endif
-    {
-        qDebug() << "Using QSytemTrayIcon";
-        trayIconMenu = new QMenu(this);
-        trayIconMenu->addAction(changeTrayIconAction);
-        trayIconMenu->addAction(restoreAction);
-        trayIconMenu->addAction(closeAction);
-        trayIcon = new CkbSystemTrayIcon(getIcon(), this);
-        trayIcon->setContextMenu(trayIconMenu);
-        trayIcon->show();
-        connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconClicked(QSystemTrayIcon::ActivationReason)));
-        connect(trayIcon, &CkbSystemTrayIcon::wheelScrolled, this, &MainWindow::handleTrayScrollEvt);
-    }
+    trayIconMenu = new QMenu(this);
+    trayIconMenu->addAction(restoreAction);
+    trayIconMenu->addAction(closeAction);
+    trayIcon = new CkbSystemTrayIcon(getIcon(), getIconName(), this);
+    trayIcon->setContextMenu(trayIconMenu);
+    trayIcon->show();
+    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconClicked(QSystemTrayIcon::ActivationReason)));
+    connect(trayIcon, &CkbSystemTrayIcon::scrollRequested, this, &MainWindow::handleTrayScrollEvt);
     toggleTrayIcon(!CkbSettings::get("Program/SuppressTrayIcon").toBool());
 
 #ifdef Q_OS_MACOS
@@ -244,9 +159,7 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(restoreAction, SIGNAL(triggered()), this, SLOT(showWindow()));
     connect(qApp, SIGNAL(applicationStateChanged(Qt::ApplicationState)), this, SLOT(stateChange(Qt::ApplicationState)));
 
-    connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(cleanup()));
-
-    ui->tabWidget->addTab(settingsWidget = new SettingsWidget(this), configLabel);
+    ui->tabWidget->addTab(settingsWidget = new SettingsWidget(this), QString(tr("Settings")));
     settingsWidget->setVersion("ckb-next " CKB_NEXT_VERSION_STR);
 
     // create daemon dialog as a QMessageBox
@@ -255,6 +168,15 @@ MainWindow::MainWindow(QWidget *parent) :
     // set the main and informative text to tell the user about the issue
     QMessageBox dialog;
     dialog.setText(tr("The ckb-next daemon is not running. This program will <b>not</b> work without it!"));
+#ifndef Q_OS_MACOS
+    QString daemonDialogText = QString(tr("Start it once with:")) +
+    "<blockquote><code>sudo systemctl start ckb-next-daemon</code></blockquote>" +
+    tr("Enable it for every boot:") +
+    "<blockquote><code>sudo systemctl enable ckb-next-daemon</code></blockquote>";
+#else
+    QString daemonDialogText = QString(tr("Start and enable it with:")) +
+    "<blockquote><code>sudo launchctl load -w /Library/LaunchDaemons/org.ckb-next.daemon.plist</code></blockquote>";
+#endif
     dialog.setInformativeText(daemonDialogText);
     dialog.setIcon(QMessageBox::Critical);
 
@@ -266,6 +188,7 @@ MainWindow::MainWindow(QWidget *parent) :
 
     signal(SIGINT, MainWindow::PosixSignalHandler);
     signal(SIGTERM, MainWindow::PosixSignalHandler);
+    signal(SIGHUP, MainWindow::PosixSignalHandler);
 
     // check, whether daemon is running
     // the daemon creates the root device path on initialization and thus it
@@ -308,9 +231,15 @@ MainWindow::MainWindow(QWidget *parent) :
         QTimer::singleShot(2000, this, &MainWindow::appleRequestHidTimer);
     }
 #endif
+#ifdef USE_XCB_EWMH
+        qRegisterMetaType<XWindowInfo>("XWindowInfo");
+        windowDetector = new XWindowDetector();
+        windowDetector->start();
+#endif
 }
 
-void MainWindow::handleTrayScrollEvt(bool up){
+void MainWindow::handleTrayScrollEvt(int delta, Qt::Orientation orientation){
+    bool up = delta > 0;
     emit trayIconScrolled(up);
 }
 
@@ -323,12 +252,7 @@ void MainWindow::checkForCkbUpdates(){
 }
 
 void MainWindow::toggleTrayIcon(bool visible){
-#ifdef USE_LIBAPPINDICATOR
-    if(useAppindicator)
-        app_indicator_set_status(indicator, visible ? APP_INDICATOR_STATUS_ACTIVE : APP_INDICATOR_STATUS_PASSIVE);
-    else
-#endif
-        trayIcon->setVisible(visible);
+    trayIcon->setVisible(visible);
 }
 
 void MainWindow::addDevice(Kb* device){
@@ -338,7 +262,7 @@ void MainWindow::addDevice(Kb* device){
             return;
     }
     // Add the keyboard
-    KbWidget* widget = new KbWidget(this, device);
+    KbWidget* widget = new KbWidget(this, device, windowDetector);
     kbWidgets.append(widget);
     // Add to tabber; switch to this device if the user is on the settings screen
     int count = ui->tabWidget->count();
@@ -373,7 +297,7 @@ void MainWindow::updateVersion(){
     // Warn if the daemon version doesn't match the GUI
     QString daemonWarning;
     if(daemonVersion != CKB_NEXT_VERSION_STR)
-        daemonWarning = "<br /><br /><b>Warning:</b> Driver version mismatch (" + daemonVersion + "). Please upgrade ckb-next" + QString(KbManager::ckbDaemonVersionF() > KbManager::ckbGuiVersionF() ? "" : "-daemon") + ". If the problem persists, try rebooting.";
+        daemonWarning = tr("<br /><br /><b>Warning:</b> Driver version mismatch (") + daemonVersion + tr("). Please upgrade ckb-next") + QString(KbManager::ckbDaemonVersionF() > KbManager::ckbGuiVersionF() ? "" : "-daemon") + tr(". If the problem persists, try rebooting.");
     if(count == 0){
 #if defined(Q_OS_MACOS) && !defined(OS_MAC_LEGACY)
         QProcess kextstat;
@@ -412,7 +336,7 @@ void MainWindow::checkFwUpdates(){
         return;
     foreach(KbWidget* w, kbWidgets){
         // Display firmware upgrade notification if a new version is available
-        float version = KbFirmware::versionForBoard(w->device->productID);
+        float version = kbfw->versionForBoard(w->device->productID);
         if(version > w->device->firmware.toFloat()){
             if(w->hasShownNewFW)
                 continue;
@@ -451,7 +375,7 @@ void MainWindow::closeEvent(QCloseEvent *event){
         return;
     }
     if(!CkbSettings::get("Popups/BGWarning").toBool()){
-        QMessageBox::information(this, "ckb-next", "ckb-next will still run in the background.\nTo close it, choose Quit from the tray menu\nor click \"Quit\" on the Settings screen.");
+        QMessageBox::information(this, "ckb-next", tr("ckb-next will still run in the background.\nTo close it, choose Quit from the tray menu\nor click \"Quit\" on the Settings screen."));
         CkbSettings::set("Popups/BGWarning", true);
     }
     hide();
@@ -461,8 +385,8 @@ void MainWindow::closeEvent(QCloseEvent *event){
 void MainWindow::timerTick(){
     // Check shared memory for changes
     if(appShare.lock()){
-        void* data = appShare.data();
-        QStringList commands = QString((const char*)data).split("\n");
+        void* shmData = appShare.data();
+        QStringList commands = QString((const char*)shmData).split("\n");
         // Restore PID, remove all other data
         snprintf((char*)appShare.data(), appShare.size(), "PID %ld", (long)getpid());
         appShare.unlock();
@@ -489,7 +413,7 @@ void MainWindow::timerTick(){
     }
     // Check for firmware updates (when appropriate)
     if(!CkbSettings::get("Program/DisableAutoFWCheck").toBool()){
-        KbFirmware::checkUpdates();
+        kbfw->checkUpdates();
         checkFwUpdates();
     }
     // Poll for setting updates
@@ -535,47 +459,50 @@ void MainWindow::quitApp(){
 }
 
 QIcon MainWindow::getIcon() {
-  // on initial launch (first time using ckb) this association checked
-  // will not be present, so force it to true, as we do not want to break default behaviour
+    // on initial launch (first time using ckb) this association checked
+    // will not be present, so force it to true, as we do not want to break default behaviour
+    if (CkbSettings::get("Program/RGBIcon", QVariant(true)).toBool())
+        return QIcon(":/img/ckb-next.png");
 
-  // If the icon is RGB then the menu should present a monochrome option
-  if (CkbSettings::get("Program/RGBIcon", QVariant(true)).toBool()){
-    changeTrayIconAction->setText("Monochrome Tray Icon");
-    connect(changeTrayIconAction, SIGNAL(triggered()), this, SLOT(changeTrayIconToMonochrome()));
-    return QIcon(":/img/ckb-next.png");
-  }
-  // If the icon is monochrome then the menu should present an RGB option
-  changeTrayIconAction->setText("RGB Tray Icon");
-  connect(changeTrayIconAction, SIGNAL(triggered()), this, SLOT(changeTrayIconToRGB()));
-  return QIcon(":/img/ckb-next-monochrome.png");
+    return QIcon(":/img/ckb-next-monochrome.png");
 }
 
-void MainWindow::changeTrayIconToMonochrome(){
-    trayIcon->setIcon(QIcon(":/img/ckb-next-monochrome.png"));
-    changeTrayIconAction->setText("RGB Tray Icon");
-    changeTrayIconAction->disconnect();
-    connect(changeTrayIconAction, SIGNAL(triggered()), this, SLOT(changeTrayIconToRGB()));
-    CkbSettings::set("Program/RGBIcon", false);
+QString MainWindow::getIconName() {
+    // Same as above but return QStrings
+    if (CkbSettings::get("Program/RGBIcon", QVariant(true)).toBool())
+        return "ckb-next";
+
+    return "ckb-next-monochrome";
 }
 
-void MainWindow::changeTrayIconToRGB(){
-    trayIcon->setIcon(QIcon(":/img/ckb-next.png"));
-    changeTrayIconAction->setText("Monochrome Tray Icon");
-    changeTrayIconAction->disconnect();
-    connect(changeTrayIconAction, SIGNAL(triggered()), this, SLOT(changeTrayIconToMonochrome()));
-    CkbSettings::set("Program/RGBIcon", true);
+void MainWindow::setTabsEnabled(bool e){
+    ui->tabWidget->tabBar()->setEnabled(e);
+    QWidget* currentWidget = ui->tabWidget->currentWidget();
+    if(!currentWidget){
+        qDebug() << "currentWidget in setTabsEnabled is null";
+        return;
+    }
+    KbWidget* kbw = dynamic_cast<KbWidget*>(currentWidget);
+    // The cast can fail if the app is quitting
+    if(!kbw)
+        return;
+    kbw->setTabBarEnabled(e);
 }
 
-void MainWindow::cleanup(){
+MainWindow::~MainWindow(){
     foreach(KbWidget* w, kbWidgets)
         delete w;
     kbWidgets.clear();
     KbManager::stop();
+    deinitAudioSubsystem();
+    delete kbfw;
     CkbSettings::cleanUp();
-}
-
-MainWindow::~MainWindow(){
-    cleanup();
+#ifdef USE_XCB_EWMH
+    if(windowDetector){
+        windowDetector->terminateEventLoop();
+        windowDetector->wait();
+    }
+#endif
     delete ui;
 }
 
@@ -590,6 +517,10 @@ void MainWindow::QSignalHandler(){
     }
 
     qDebug() << "\nSignal" << sig << "caught. Quitting...";
+    if(sig == SIGHUP){
+        // Restart, but with a delay
+        QProcess::startDetached(QCoreApplication::applicationFilePath(), QStringList() << "-b" << "-d");
+    }
     this->quitApp();
     sigNotifier->setEnabled(true);
 }
@@ -603,14 +534,18 @@ void MainWindow::PosixSignalHandler(int signal){
 void MainWindow::checkedForNewVer(QString ver, QString changelog){
 #ifndef DISABLE_UPDATER
     if(!ver.isEmpty()) {
-        settingsWidget->setUpdateButtonText("Update to v" + ver);
+        settingsWidget->setUpdateButtonText(tr("Update to v") + ver);
         showWindow();
         CkbUpdaterDialog updDialog(ver, changelog, this);
         updDialog.exec();
     } else {
-        settingsWidget->setUpdateButtonText("Up to date");
+        settingsWidget->setUpdateButtonText(tr("Up to date"));
     }
     updater->deleteLater();
     settingsWidget->enableUpdateButton();
 #endif
 }
+
+void MainWindow::syncTrayIcon(){
+    trayIcon->setIcon(getIcon(), getIconName());
+}
